package com.agilex.healthcare.directscheduling.dataservice;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import agilex.commons.helper.PropertyHelper;

import com.agilex.healthcare.directscheduling.datalayer.SchedulingDataLayer;
import com.agilex.healthcare.directscheduling.domain.AppointmentSlotFilter;
import com.agilex.healthcare.directscheduling.domain.AppointmentTimeSlot;
import com.agilex.healthcare.directscheduling.domain.AppointmentTimeSlots;
import com.agilex.healthcare.directscheduling.domain.BookAppointment;
import com.agilex.healthcare.directscheduling.domain.BookAppointmentEmail;
import com.agilex.healthcare.directscheduling.domain.BookedAppointment;
import com.agilex.healthcare.directscheduling.domain.BookedAppointmentCollection;
import com.agilex.healthcare.directscheduling.domain.BookedAppointments;
import com.agilex.healthcare.directscheduling.domain.CancelAppointment;
import com.agilex.healthcare.directscheduling.domain.CancelAppointmentEmail;
import com.agilex.healthcare.directscheduling.domain.CancelReasons;
import com.agilex.healthcare.directscheduling.domain.DirectSchedulingFacilityProvider;
import com.agilex.healthcare.directscheduling.domain.DirectSchedulingFacilityProviders;
import com.agilex.healthcare.directscheduling.domain.VARBookedAppointmentCollections;
import com.agilex.healthcare.directscheduling.linkbuilder.DSCancelAppointmentLinkBuilder;
import com.agilex.healthcare.directscheduling.message.EmailDefaults;
import com.agilex.healthcare.directscheduling.message.EmailNotificationMessage;
import com.agilex.healthcare.directscheduling.message.EmailNotificationPublisher;
import com.agilex.healthcare.directscheduling.utils.DateHelper;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifiers;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterQueryParameters;
import com.agilex.healthcare.mobilehealthplatform.utils.QueryParameters;
import com.agilex.va.mpi.PatientCorrelationGateway;

@Primary
@Service
@Scope( BeanDefinition.SCOPE_SINGLETON )
public class DSDataService {	
	
		@Resource
		private SchedulingDataLayer schedulingDataLayer ;
	
		@Resource
		protected PatientCorrelationGateway patientCorrelationGateway ;
		
	    @Resource
	    private EmailNotificationPublisher publisher;
	    
	    @Resource
	    private PropertyHelper propertyHelper;
	    
	    private static final String CANCEL_CODE = "PC";
		private static final String GENERIC_BOOKING_ERROR = "An error occurred while booking the appointment.";
		private static final String EMAIL_ERROR = "An error occured while sending email.";
		private static final List<String> CANCEL_CODE_LIST =  Arrays.asList("NO-SHOW", 
																	"CANCELLED BY CLINIC", 
																	"NO-SHOW & AUTO RE-BOOK",
																	"CANCELLED BY CLINIC & AUTO RE-BOOK",
																	"INPATIENT APPOINTMENT",
																	"CANCELLED BY PATIENT",
																	"CANCELLED BY PATIENT & AUTO-REBOOK",
																	"NO ACTION TAKEN");
		
		public DSDataService( ) { 
		}
		
		private AppointmentSlotFilter getAvailableAppointmentSlotsCriteria(UriInfo uriInfo, String siteCode, String clinicId){
	        DateFilter dateFilter = DateFilterFactory.createFilterFromUri(uriInfo);

	        AppointmentSlotFilter appointmentSlotFilter = new AppointmentSlotFilter();
	        appointmentSlotFilter.setClinicId(clinicId);
	        appointmentSlotFilter.setSiteCode(siteCode);
	        appointmentSlotFilter.setStartDate(dateFilter.getStartDate());
	        appointmentSlotFilter.setEndDate(dateFilter.getEndDate());

	        return appointmentSlotFilter;
	    }
		
		private AppointmentSlotFilter getBookedAppointmentFilterCriteria(UriInfo uriInfo, String siteCode){
	        QueryParameters queryParameters = new QueryParameters(uriInfo.getRequestUri().getQuery());     
	        DateFilterQueryParameters parameters = new DateFilterQueryParameters(queryParameters);

	        AppointmentSlotFilter appointmentSlotFilter = new AppointmentSlotFilter();
	        appointmentSlotFilter.setSiteCode(siteCode);
	        appointmentSlotFilter.setStartDate(DateHelper.determineStartDate(null, parameters));
	        return appointmentSlotFilter;
	    }
		
		private PatientIdentifier getVistaPatientIdentifier(PatientIdentifier edipiPatientIdentifier, String siteCode) {
			PatientIdentifiers patientIdentifiers = patientCorrelationGateway.getCorrespondIdentifiers(edipiPatientIdentifier);
	        if(patientIdentifiers != null && patientIdentifiers.size() > 0){
	            for (PatientIdentifier patientIdentifier : patientIdentifiers){
	                String assigningAuthority = "dfn-" + siteCode;
	                if(patientIdentifier.getAssigningAuthority().equals(assigningAuthority)){
	                    return patientIdentifier;
	                }
	            }
	        }
	        return null;
		}

		public Response cancelAppointmentForPatient(CancelAppointment cancelAppointment,PatientIdentifier patientIdentifier, String siteCode) {
			cancelAppointment.setCancelCode(CANCEL_CODE);
			
			PatientIdentifier icnPatientIdentifier = getIcnIdentifiers(patientIdentifier);
			if (icnPatientIdentifier == null) {
				return (null);
			}
			
			cancelAppointment.setPatientIdentifier(icnPatientIdentifier);
			CancelAppointment canceledAppointment = schedulingDataLayer.cancelAppointmentForPatient(cancelAppointment, siteCode);
			if(isAppointmentCanceledSuccessfully(cancelAppointment, canceledAppointment)){
	            return Response.ok().build();
	        }
			
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(GENERIC_BOOKING_ERROR).build();
		}
		
		public CancelReasons getCancelReasonList(PatientIdentifier patientIdentifier, String siteCode){
			PatientIdentifier icnPatientIdentifier = getIcnIdentifiers(patientIdentifier);
			if (icnPatientIdentifier == null) {
				return (null);
			}
			
			CancelReasons cancelReasons = schedulingDataLayer.getCancelReasonList(icnPatientIdentifier,siteCode);
			return cancelReasons;
		}
		
		public VARBookedAppointmentCollections getPatientBookedAppointments(UriInfo uriInfo, PatientIdentifier patientIdentifier,String siteCode,String clinicIds){
			VARBookedAppointmentCollections bookedAppointmentCollections = new VARBookedAppointmentCollections( ) ; 
			
			PatientIdentifier icnPatientIdentifier = getIcnIdentifiers(patientIdentifier);
			if (icnPatientIdentifier == null) {
				return (null);
			}
			
			// This object is nothing more than a container of BookedAppointmentCollections. Each BookedAppointmentCollection contains
			// a collection of BookedAppointments and a collection name for that specific collection.
			// This is the main container that holds all BookedAppointmentCollection objects. Each BookedAppointmentCollection object
			// contains a collection type name and a collection of BookedAppointments.
			Collection< BookedAppointmentCollection > bookedAppointmentCollectionContainer = bookedAppointmentCollections.getBookedAppointmentCollections( ) ; 
			
			if( icnPatientIdentifier != null ) {

				AppointmentSlotFilter appointmentSlotFilter = getBookedAppointmentFilterForPatient( uriInfo , siteCode , icnPatientIdentifier.getUniqueId() ) ;
				
				appointmentSlotFilter.setPatientIdentifier( icnPatientIdentifier ) ;
				BookedAppointments bookedAppointments = schedulingDataLayer.getPatientBookedAppointments( appointmentSlotFilter ) ;

				DSCancelAppointmentLinkBuilder linkBuilder = new DSCancelAppointmentLinkBuilder( uriInfo.getBaseUri( ) ) ;
				bookedAppointments = linkBuilder.fillLinks( bookedAppointments , patientIdentifier , siteCode ) ;

				Collection< BookedAppointment > allBookedAppointmentsCollection = bookedAppointments.getBookedAppointment( ) ;

				String primaryCareClinicId = clinicIds;

				// The removeBookedAppointmentsByClinicId method will REMOVE those appointments from the passed collection
				// and returns them. Therefore, keep in mind this modifies the original passed collection.
				Collection< BookedAppointment > primaryCareBookedAppointments = 
						removeBookedAppointmentsByClinicId( primaryCareClinicId , allBookedAppointmentsCollection ) ;
				
				if( primaryCareBookedAppointments != null ) { 

					// Now let's add the primary care appointments to a new BookedAppointmentCollection and set the name.
					BookedAppointmentCollection primaryCareBookedAppointmentCollection = new BookedAppointmentCollection( ) ; 
					primaryCareBookedAppointmentCollection.setCollectionName( "PRIMARY_CARE" );
					primaryCareBookedAppointmentCollection.setBookedAppointments( primaryCareBookedAppointments ) ; 

					// Add the new primary care BookedAppointmentCollection to the container holding all BookedAppointmentCollections.
					bookedAppointmentCollectionContainer.add( primaryCareBookedAppointmentCollection ) ; 
				}

				// Check the full list of booked appointments because, depending on how many have been removed, there
				// may not be any remaining.
				if( allBookedAppointmentsCollection != null && allBookedAppointmentsCollection.size( ) > 0 ) { 
					
					// Now create a BookedAppointmentCollection of all NON primary care appointments.
					BookedAppointmentCollection nonPrimaryCareBookedAppointmentCollection = new BookedAppointmentCollection( ) ; 
					nonPrimaryCareBookedAppointmentCollection.setCollectionName( "OTHER" ) ;
					nonPrimaryCareBookedAppointmentCollection.setBookedAppointments( allBookedAppointmentsCollection ) ; 

					// Add the new non primary care BookedAppointmentCollection to the container holding all BookedAppointmentCollections.
					bookedAppointmentCollectionContainer.add( nonPrimaryCareBookedAppointmentCollection ) ; 				
				}
			}
			
			// This is returning a bookedAppointmentCollections, which contains a collection of "BookedAppointmentCollection". 
			// Each BookedAppointmentCollection has a collection name, signifying the type (like Primary Care or Other), and a
			// collection of BookedAppointment objects.
			return( bookedAppointmentCollections ) ; 
		}
		
		public Response bookNewAppointmentForPatient(BookAppointment bookAppointment,
														PatientIdentifier patientIdentifier, String siteCode) { 
			PatientIdentifier icnPatientIdentifier = getIcnIdentifiers(patientIdentifier);
			if (icnPatientIdentifier == null) {
				return (null);
			}
			
			bookAppointment.setPatientIdentifier(icnPatientIdentifier);
	        BookAppointment newBookedAppointment = new BookAppointment();
	        try{
	            newBookedAppointment = schedulingDataLayer.bookPatientAppointment(bookAppointment, siteCode);
	        }catch(Exception e){
	            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(GENERIC_BOOKING_ERROR).build();
	        }
	        if(newBookedAppointment.getError() != null && newBookedAppointment.getError().length() > 0){
	            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(newBookedAppointment.getError()).build();
	        }
	        if(isAppointmentSavedCorrectly(bookAppointment, newBookedAppointment)){
	            return Response.ok().build();
	        }
	        return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(GENERIC_BOOKING_ERROR).build();	
		}
		
		private Collection< BookedAppointment > removeBookedAppointmentsByClinicId( final String clinicIds , Collection< BookedAppointment > bookedAppointments ) { 
			
			if( clinicIds == null || bookedAppointments == null || bookedAppointments.size( ) == 0 ) { 
				
				return( bookedAppointments ) ; 
			}
			
			String[] ClinicIdsArray = StringUtils.split(clinicIds, ",");
			List<String> clinicIdsAsList =  Arrays.asList(ClinicIdsArray);
			
			Collection< BookedAppointment > removeBookedAppointmentsCollection = new ArrayList< BookedAppointment >( ) ; 
			
			for( BookedAppointment singleBookedAppointment : bookedAppointments ) { 
				
				if( clinicIdsAsList.contains( singleBookedAppointment.getClinic( ).getId( ) ) ) { 
					
					removeBookedAppointmentsCollection.add( singleBookedAppointment ) ; 
				}
			}
			
			if( removeBookedAppointmentsCollection.size( ) > 0 ) { 
				
				bookedAppointments.removeAll( removeBookedAppointmentsCollection ) ; 
			}
			
			return( removeBookedAppointmentsCollection.size( ) > 0 ? removeBookedAppointmentsCollection : null ) ; 
		}
		
	 private AppointmentSlotFilter getBookedAppointmentFilterForPatient(UriInfo uriInfo, String siteCode, String patientId){
	        AppointmentSlotFilter appointmentSlotFilter = getBookedAppointmentFilterCriteria(uriInfo, siteCode);
	        appointmentSlotFilter.setPatientId(patientId);
	        return appointmentSlotFilter;
	 }
	 
	 private boolean isAppointmentSavedCorrectly(BookAppointment requestedAppointment, BookAppointment savedAppointment){
	        return (requestedAppointment.getDateTime().equals(savedAppointment.getDateTime())
	                    && requestedAppointment.getClinicId().equals(savedAppointment.getClinicId())
	                    && requestedAppointment.getPurpose().equals(savedAppointment.getPurpose()));
	    }
	 
	 private boolean isAppointmentCanceledSuccessfully(CancelAppointment originalObject, CancelAppointment returnObject){
	        return (originalObject.getClinicId().equals(returnObject.getClinicId())
	                && originalObject.getAppointmentTime().equals(returnObject.getAppointmentTime())
	                && originalObject.getCancelCode().equals(returnObject.getCancelCode())
	                && originalObject.getCancelReason().equals(returnObject.getCancelReason()));
	    }
	
	public Response sendEmailNotificationForBooking(final BookAppointmentEmail bookAppointmentEmail){
		if(bookAppointmentEmail.getDateTime() == null || bookAppointmentEmail.getFacilityName() == null || 
				bookAppointmentEmail.getClinic() == null || bookAppointmentEmail.getEmailAddress() == null){
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(EMAIL_ERROR).build();
		}
		StringBuilder body = new StringBuilder();
		body.append("Your appointment is booked for")
			.append("\n")
			.append("Date: " + bookAppointmentEmail.getDateTime())
			.append("\n")
			.append("Facility Name: " + bookAppointmentEmail.getFacilityName())
			.append("\n")
			.append("Clinic Name: " + bookAppointmentEmail.getClinic());
		return sendEmailNotification(bookAppointmentEmail.getEmailAddress(),body.toString());
	}
	
	public Response sendEmailNotificationForCancelling(final CancelAppointmentEmail cancelAppointmentEmail){
		if(cancelAppointmentEmail.getAppointmentTime() == null || cancelAppointmentEmail.getFacility().getName() == null || 
				cancelAppointmentEmail.getClinicName() == null || cancelAppointmentEmail.getEmailAddress() == null){
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(EMAIL_ERROR).build();
		}
		StringBuilder body = new StringBuilder();
		body.append("Your appointment is cancelled for")
			.append("\n")
			.append("Date: " + cancelAppointmentEmail.getAppointmentTime())
			.append("\n")
			.append("Facility Name: " + cancelAppointmentEmail.getFacility().getName())
			.append("\n")
			.append("Clinic Name: " + cancelAppointmentEmail.getClinicName());
		return sendEmailNotification(cancelAppointmentEmail.getEmailAddress(),body.toString());
	}
	
    private Response sendEmailNotification(final String emailAddress,final String body) {
        EmailNotificationMessage message = null;
        message = new EmailNotificationMessage(emailAddress,
                    new EmailDefaults(propertyHelper.getProperty("email.notifications.fromAddress"),
                            propertyHelper.getProperty("email.notifications.replyToAddress"),
                            propertyHelper.getProperty("email.notifications.subject"),
                            body));
        try{
            publisher.send(message);
        }
        catch(Exception e){
        	return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(EMAIL_ERROR).build();
        }
        return Response.ok().build();
      }
		
		public Collection<AppointmentTimeSlots> getFilteredAppointmentSlots(UriInfo uriInfo,PatientIdentifier patientIdentifier,String siteCode,String clinicIds) {	
			Collection<AppointmentTimeSlots> collectionOfApointmentTimeSlots = null;
			
			PatientIdentifier icnPatientIdentifier = getIcnIdentifiers(patientIdentifier);
			if (icnPatientIdentifier == null) {
				return (null);
			}
			
			if(clinicIds != null && clinicIds.length() >= 1){
					AppointmentSlotFilter appointmentSlotFilterForPatient = getBookedAppointmentFilterForPatient(uriInfo,siteCode,icnPatientIdentifier.getUniqueId()) ;			
					appointmentSlotFilterForPatient.setPatientIdentifier(icnPatientIdentifier);
				
					List<String> clinicIdList = new ArrayList<String>(Arrays.asList(StringUtils.split(clinicIds ,",")));
					
					collectionOfApointmentTimeSlots = new ArrayList<AppointmentTimeSlots>();
						
					Collection<Date> primaryCareBookedAppointmentDates = getAllPrimaryCareBookedAppointmentDates(appointmentSlotFilterForPatient,clinicIdList); 
					
					for(String clinicId:clinicIdList) { 
						AppointmentSlotFilter appointmentSlotFilter = getAvailableAppointmentSlotsCriteria(uriInfo,siteCode,clinicId);			
						appointmentSlotFilter.setPatientIdentifier(icnPatientIdentifier);
						AppointmentTimeSlots appointmentTimeSlots = schedulingDataLayer.getClinicSchedulingDetail(appointmentSlotFilter) ;					
						
						Map<String, List<AppointmentTimeSlot>> appointmentTimeSlotsByClinicByDay = filterAppointmentTimeSlotsByCriteria(appointmentTimeSlots);
						if(appointmentTimeSlotsByClinicByDay != null && appointmentTimeSlotsByClinicByDay.values().size() > 0){
							appointmentTimeSlots.removeAll(appointmentTimeSlots.getAppointmentTimeSlot());
							appointmentTimeSlots.addAll(filterAppointmentByBookedAppointments(appointmentTimeSlotsByClinicByDay,primaryCareBookedAppointmentDates));
							collectionOfApointmentTimeSlots.add(appointmentTimeSlots);
						}												
					}	
				}
			return(collectionOfApointmentTimeSlots);			
		}
		
		protected List<String> getIcnIdentifiers(PatientIdentifiers correspondIdentifiers) { 
			List<String> foundIcnIdentifiers = new ArrayList<String>();
	        for (PatientIdentifier correspondIdentifier : correspondIdentifiers) {
	            if ("ICN".equalsIgnoreCase(correspondIdentifier.getAssigningAuthority())){
	                foundIcnIdentifiers.add(correspondIdentifier.getUniqueId());
	            }
	        }
	        return foundIcnIdentifiers;
	    }
		
		protected PatientIdentifier getIcnIdentifiers(PatientIdentifier patientIdentifier) { 
			PatientIdentifiers correspondIdentifiers = patientCorrelationGateway.getCorrespondIdentifiers(patientIdentifier);
	 		List<String> icnIdentifiers = getIcnIdentifiers(correspondIdentifiers);
	 		return ((icnIdentifiers==null || icnIdentifiers.isEmpty())?null:new PatientIdentifier("ICN",icnIdentifiers.get(0)));
	    }
		
		protected List<String> getDfnIdentifiers(PatientIdentifiers patientIdentifers,DirectSchedulingFacilityProviders dsfps) { 
			List<String> foundDfnIdentifiers = new ArrayList<String>();
			if(dsfps != null && !dsfps.getFacilityProviderList().isEmpty()){
				Iterator<DirectSchedulingFacilityProvider> iterator = dsfps.getFacilityProviderList().iterator();
				while(iterator.hasNext()){
					String assigningAuthority = "DFN-"+iterator.next().getFacilityId();
			        for (PatientIdentifier correspondIdentifier : patientIdentifers) {
			        	 if (StringUtils.equalsIgnoreCase(correspondIdentifier.getAssigningAuthority(), assigningAuthority)){
			        		 foundDfnIdentifiers.add(correspondIdentifier.getUniqueId());
				          }
			        }
			    }
			}
			return foundDfnIdentifiers;
	    }
		
		private AppointmentTimeSlots filterAppointmentByBookedAppointments(Map<String, List<AppointmentTimeSlot>> appointmentTimeSlotsByClinicByDay,Collection<Date> primaryCareBookedAppointmentDates) { 			
			if(primaryCareBookedAppointmentDates != null && !primaryCareBookedAppointmentDates.isEmpty()){
				for(Date bookedAppointmentDate:primaryCareBookedAppointmentDates) { 
					String key = DateHelper.format(bookedAppointmentDate,DateHelper.REPORT_DATE_FORMAT);
					if(appointmentTimeSlotsByClinicByDay.containsKey(key)){
						appointmentTimeSlotsByClinicByDay.remove(key);
					}
				}
			}
			AppointmentTimeSlots tempAppointmentTimeSlots = null;
			if(appointmentTimeSlotsByClinicByDay != null && !appointmentTimeSlotsByClinicByDay.values().isEmpty()){
				tempAppointmentTimeSlots = new AppointmentTimeSlots();
				for(List<AppointmentTimeSlot> listOfAppointmentTimeSlot : appointmentTimeSlotsByClinicByDay.values()){
					tempAppointmentTimeSlots.getAppointmentTimeSlot().addAll(listOfAppointmentTimeSlot);
				}
			}
			return(tempAppointmentTimeSlots) ;  
		}
		
		private Map<String, List<AppointmentTimeSlot>> filterAppointmentTimeSlotsByCriteria(AppointmentTimeSlots appointmentTimeSlotsByClinic) { 
			int timeDurationThreshold = 1800000 ; 
			if(appointmentTimeSlotsByClinic != null && !appointmentTimeSlotsByClinic.getAppointmentTimeSlot().isEmpty()){		
				Map<String, List<AppointmentTimeSlot>> appointmentTimeSlotsByClinicByDay = new HashMap<String, List<AppointmentTimeSlot>>();		
				
				Iterator<AppointmentTimeSlot> it = appointmentTimeSlotsByClinic.getAppointmentTimeSlot().iterator( ) ;			
				while(it.hasNext()) { 				
					AppointmentTimeSlot appointmentTimeSlot = it.next( ) ; 				
					if(!appointmentTimeSlot.isAvailability()) {
						continue ;
					}
					if(!isAppointmentNomenclatureValid(appointmentTimeSlot)) { 					
						continue ; 
					}
					if(appointmentTimeSlot.getEndDateTime().getTime() - appointmentTimeSlot.getStartDateTime().getTime() > timeDurationThreshold) { 				
						continue ; 
					}
					
					String key = DateHelper.format(appointmentTimeSlot.getStartDateTime(),DateHelper.REPORT_DATE_FORMAT);				
					if(appointmentTimeSlotsByClinicByDay.containsKey(key)){
						appointmentTimeSlotsByClinicByDay.get(key).add(appointmentTimeSlot) ;
					}
					else{
						List<AppointmentTimeSlot> tempAppointmentTimeSlot = new  ArrayList<AppointmentTimeSlot>() ; 
						tempAppointmentTimeSlot.add(appointmentTimeSlot);
						appointmentTimeSlotsByClinicByDay.put(key,tempAppointmentTimeSlot) ; 
					}				
				}	
				return(appointmentTimeSlotsByClinicByDay);
			}
			return null;
		}
	
		private boolean isAppointmentNomenclatureValid(AppointmentTimeSlot appointmentTimeSlot) { 			
			if(appointmentTimeSlot!=null){
				String bookingStatus = appointmentTimeSlot.getBookingStatus( ) ; 
				char ch = bookingStatus.length( ) > 0 ? bookingStatus.charAt( 0 )  : '0' ; 			
				if( ( ch>='1' ) && ( ch <= '9' ) || ( ch >= 106 ) && ( ch <= 122 ) ) { 					
					return( true ) ; 
				} 			
				return( false );
			}
			return( false );
		}
			
		private Collection<Date> getAllPrimaryCareBookedAppointmentDates(AppointmentSlotFilter appointmentSlotFilter,List<String> clinicIdList) { 			
			BookedAppointments bookedAppointments = schedulingDataLayer.getPatientBookedAppointments(appointmentSlotFilter) ;			
				
			Collection<Date> bookedDates = null;
			
			if(bookedAppointments != null && !bookedAppointments.getBookedAppointment().isEmpty()){
				bookedDates = new ArrayList<Date>();
				/*
				 * To remove appointments slots from current date as well. User can't book appointment for current day.
				 */
				bookedDates.add(DateHelper.getToday());
				Iterator<BookedAppointment> bkIterator = bookedAppointments.getBookedAppointment().iterator(); 
				while(bkIterator.hasNext()) { 				
					BookedAppointment ba = bkIterator.next() ; 				
					if(clinicIdList.contains(ba.getClinic().getId()) && !CANCEL_CODE_LIST.contains((ba.getCurrentStatus()))){
							bookedDates.add(ba.getAppointmentTime());
					}
				}
			}
			return(bookedDates); 
		}
}