package com.agilex.healthcare.directscheduling.resource;

import java.util.Collection;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.directscheduling.dataservice.DSDataService;
import com.agilex.healthcare.directscheduling.dataservice.DSFacilityProviderDataService;
import com.agilex.healthcare.directscheduling.domain.BookAppointmentEmail;
import com.agilex.healthcare.directscheduling.domain.CancelAppointmentEmail;
import com.agilex.healthcare.directscheduling.domain.DirectSchedulingFacilityProviders;
import com.agilex.healthcare.directscheduling.domain.VARBookedAppointmentCollections;
import com.agilex.healthcare.directscheduling.domain.facility.DSFacilities;
import com.agilex.healthcare.directscheduling.linkbuilder.DirectSchedulingResourceDirectoryBuilder;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.directscheduling.domain.AppointmentTimeSlots;
import com.agilex.healthcare.directscheduling.domain.BookAppointment;
import com.agilex.healthcare.directscheduling.domain.CancelAppointment;
import com.agilex.healthcare.directscheduling.domain.CancelReasons;
import com.agilex.healthcare.veteranappointment.resources.AbstractUserResource;

@Path("/direct-scheduling")
@Component
@Scope("request")
public class DSResource extends AbstractUserResource{
	
	@Resource
	DSDataService dsDataService;
	
	@Resource
	DSFacilityProviderDataService dsFacilityProviderDataService ;
	
	@Resource
	DirectSchedulingResourceDirectoryBuilder dsResourceDirectoryBuilder;
	
	/**
	 * This REST service is used to get resource directory for direct scheduling.
	 * @param uriInfo
	 * @param siteCode
	 * @return ResourceDirectory
	 */
	@GET
	@Path("/directscheduling-resource-directory")
	@Produces({"application/xml","application/json"})
	public ResourceDirectory getResourceDirectory(@Context UriInfo uriInfo, @QueryParam("siteCode") String siteCode){
		MhpUser user = getCurrentUser();
		ResourceDirectory rd = dsResourceDirectoryBuilder.getResourceDirectory(uriInfo.getBaseUri(), siteCode, user.getUserIdentifier().getAssigningAuthority(), user.getUserIdentifier().getUniqueId());
		return rd;
	}
	
	/**
	 * This REST service is used to fetch all the appointment slots for selected facility from vista via scheduling manager for logged in patient/veteran
	 * @param uriInfo
	 * @param request
	 * @param headers
	 * @param siteCode
	 * @param assigningAuthority
	 * @param patientId
	 * @param startDate
	 * @param endDate
	 * @param clinicIds
	 * @return Collection<AppointmentTimeSlots>
	 */
	@GET
	@Path("/site/{site-code}/patient/{assigning-authority}/{patient-id}/available-appointment-slots")
	@Produces({"application/xml","application/json"})
	public Collection<AppointmentTimeSlots> getAppointmentSlots(@Context UriInfo uriInfo, @Context HttpServletRequest request, 
			@Context HttpHeaders headers, @PathParam("site-code") String siteCode,
			@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId,
			@QueryParam("startDate") String startDate, @QueryParam("endDate") String endDate,
			@QueryParam( "clinicIds" ) String clinicIds){
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		Collection<AppointmentTimeSlots> appointmentTimeSlots = dsDataService.getFilteredAppointmentSlots(uriInfo,patientIdentifier,siteCode,clinicIds) ;
		return appointmentTimeSlots ;
	}
	
	/**
	 * This REST service is used to update the booked appointment status to cancel in selected facility in vista via scheduling manager for logged in patient/veteran
	 * @param cancelAppointment
	 * @param siteCode
	 * @param assigningAuthority
	 * @param patientId
	 * @param uriInfo
	 * @param headers
	 * @param request
	 * @return Response
	 */
	@PUT
	@Path("/site/{site-code}/patient/{assigning-authority}/{patient-id}/cancel-appointment")
	@Produces({"application/xml","application/json"})
	@Consumes({"application/json","application/xml"})
	public Response cancelPatientAppointment(CancelAppointment cancelAppointment, @PathParam("site-code") String siteCode,
			@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, 
			@Context UriInfo uriInfo, @Context HttpHeaders headers, @Context HttpServletRequest request){
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		Response response = dsDataService.cancelAppointmentForPatient(cancelAppointment, patientIdentifier,siteCode);
		return response;
	}
	
	/**
	 * This REST service is used to save appointment for selected facility in vista via scheduling manager for logged in patient/veteran
	 * @param bookAppointment
	 * @param siteCode
	 * @param assigningAuthority
	 * @param patientId
	 * @param uriInfo
	 * @param headers
	 * @param request
	 * @return Response
	 */
	@POST
	@Path( "/site/{site-code}/patient/{assigning-authority}/{patient-id}/booked-appointments" )
	@Produces({"application/xml","application/json"})
	@Consumes({"application/json","application/xml"})
	public Response bookNewAppointment(BookAppointment bookAppointment, 
			@PathParam("site-code") String siteCode, 
			@PathParam("assigning-authority") String assigningAuthority, 
			@PathParam("patient-id") String patientId, 
			@Context UriInfo uriInfo, @Context HttpHeaders headers, @Context HttpServletRequest request)  {
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority,patientId) ;	
		Response response = dsDataService.bookNewAppointmentForPatient(bookAppointment,patientIdentifier,siteCode);
		return response;
	}

	/**
	 * This REST service is used to fetch cancel reason list for selected facility from vista via scheduling manager for logged in patient/veteran
	 * @param siteCode
	 * @param uriInfo
	 * @param headers
	 * @param request
	 * @return CancelReasons
	 */
	@GET
    @Produces({ "application/json", "application/xml" })
    @Path("/site/{site-code}/patient/{assigning-authority}/{patient-id}/cancel-reasons-list")
    public CancelReasons getCancelReasonsDropDownList(@PathParam("site-code") String siteCode, 
    													@Context UriInfo uriInfo, 
    													@Context HttpHeaders headers,
    													@Context HttpServletRequest request,
    													@PathParam("assigning-authority") String assigningAuthority, 
    													@PathParam("patient-id") String patientId){
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority,patientId) ;	
		CancelReasons cancelReasons = dsDataService.getCancelReasonList(patientIdentifier,siteCode);
		return cancelReasons;
    }
	
	/**
	 * This REST service is used to fetch all booked appointments for selected facility from vista via scheduling manager for logged in patient/veteran.
	 * Service returns collection with booked appointments by "PRIMART CARE" and "OTHER"
	 * @param uriInfo
	 * @param siteCode
	 * @param patientId
	 * @param assigningAuthority
	 * @param startDate
	 * @param clinicIds
	 * @return VARBookedAppointmentCollections
	 */
	@GET
    @Produces({"application/xml", "application/json"})
    @Path("/site/{site-code}/patient/{assigning-authority}/{patient-id}/booked-appointments")
    public VARBookedAppointmentCollections getPendingAppointmentsForPatient(@Context UriInfo uriInfo,
                                                                          @PathParam("site-code") String siteCode, @PathParam("patient-id") String patientId,
                                                                          @PathParam("assigning-authority") String assigningAuthority, @QueryParam("startDate") String startDate,
                                                                          @QueryParam( "clinicIds" ) String clinicIds){
		
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		VARBookedAppointmentCollections bookedAppointments = dsDataService.getPatientBookedAppointments(uriInfo,patientIdentifier,siteCode,clinicIds);
		return bookedAppointments;
    }
	
	/**
	 * This REST service is used to logout MDWS session open for selected facility for logged in patient/veteran
	 * @param uriInfo
	 * @param request
	 * @param headers
	 * @param siteCode
	 * @param patientId
	 * @param assigningAuthority
	 * @return Response
	 */
	@GET
    @Produces({"application/xml", "application/json"})
    @Path("/site/{site-code}/logout")
    public Response logout(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers,
                                                                          @PathParam("site-code") String siteCode, @PathParam("patient-id") String patientId,
                                                                          @PathParam("assigning-authority") String assigningAuthority){
		return Response.ok().build();
    }
	
	/**
	 * This REST service sends email confirmation after appointment is booked
	 * @param bookAppointmentEmail
	 * @param uriInfo
	 * @return Response
	 */
	@PUT
	@Produces({"application/xml","application/json"})
	@Path("/booking-confirmation")
	public Response bookingConfirmation(BookAppointmentEmail bookAppointmentEmail, @Context UriInfo uriInfo){
		return dsDataService.sendEmailNotificationForBooking(bookAppointmentEmail);
	}
	
	/**
	 * This REST service sends email confirmation after appointment is cancelled
	 * @param cancelApppointmentEmail
	 * @param uriInfo
	 * @return Response
	 */
	@PUT
	@Produces({"application/xml","application/json"})
	@Path("cancel-confirmation")
	public Response cancelConfirmation(CancelAppointmentEmail cancelApppointmentEmail, @Context UriInfo uriInfo){
		return dsDataService.sendEmailNotificationForCancelling(cancelApppointmentEmail);
	}	
	
	/**
	 * This REST service is used to fetch PACT relationship with facilities, providers and clinics associated for logged in patient/veteran
	 * @param uriInfo
	 * @param httpHeaders
	 * @param assigningAuthority
	 * @param patientId
	 * @return DirectSchedulingFacilityProviders
	 */
	@GET
	@Produces( { "application/xml" , "application/json" } )
	@Path( "/patient/{assigning-authority}/{patient-id}/facility-provider" )
	public DirectSchedulingFacilityProviders getPactFacilityProviders( @Context UriInfo uriInfo ,
			@Context HttpHeaders httpHeaders ,
			@PathParam( "assigning-authority" ) String assigningAuthority ,
			@PathParam( "patient-id" ) String patientId ) {

		PatientIdentifier patientIdentifer = new PatientIdentifier( assigningAuthority , patientId ) ;
		DirectSchedulingFacilityProviders facilityProvider = dsFacilityProviderDataService.getPactFacilityMembersForPatient( patientIdentifer ) ;
		return facilityProvider ;
	}
	
	
	@GET
	@Produces({"application/xml","application/json"})
	@Path( "/patient/{assigning-authority}/{patient-id}/facility-provider-all" )
	public DSFacilities getFacilityProvidersAll(@Context UriInfo uriInfo, @Context HttpServletRequest request, 
			@Context HttpHeaders headers, @PathParam("site-code") String siteCode,
			@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId ){
		
		PatientIdentifier patientIdentifier = new PatientIdentifier( assigningAuthority , patientId ) ;
		DSFacilities facilities = dsFacilityProviderDataService.getAllFacilityProvidersForPatient( patientIdentifier ) ; 
		return( facilities ) ; 
	}	
}