package com.agilex.healthcare.veteranappointment.dataservice;

import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.Facilities;
import com.agilex.healthcare.mobilehealthplatform.domain.Facility;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientData;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ValidationError;
import com.agilex.healthcare.mobilehealthplatform.domain.ValidationResult;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.enumeration.RunningMode;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.agilex.healthcare.veteranappointment.datalayer.appointment.AppointmentRequestDataLayer;
import com.agilex.healthcare.veteranappointment.datalayer.appointment.AppointmentRequestFilter;
import com.agilex.healthcare.veteranappointment.datalayer.facility.FacilityDataService;
import com.agilex.healthcare.veteranappointment.domain.*;
import com.agilex.healthcare.veteranappointment.enumeration.AppointmentRequestStatus;
import com.agilex.healthcare.veteranappointment.enumeration.AppointmentRequestType;
import com.agilex.healthcare.veteranappointment.exception.InvalidAppointmentRequestException;
import com.agilex.healthcare.veteranappointment.exception.InvalidAppointmentRequestMessageException;
import com.agilex.healthcare.veteranappointment.validator.AppointmentRequestMessageValidator;
import com.agilex.healthcare.veteranappointment.validator.AppointmentRequestMessageValidatorFactory;
import com.agilex.healthcare.veteranappointment.validator.AppointmentRequestValidator;
import com.agilex.healthcare.veteranappointment.validator.AppointmentRequestValidatorFactory;

import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.WebApplicationException;

import java.util.Date;
import java.util.List;

@Component
public class AppointmentRequestDataService {
	
	@Resource
	AppointmentRequestDataLayer appointmentRequestDataLayer;
	
	@Resource
	FacilityDataService facilityDataService;
	
	private static final int EarliestTimeVeteranCanRequestAppointmentInHours = 96;
	private static final int FarthestTimeVeteranCanRequestAppointmentInDays = 120;
    private static final int messageLimitPerSender = 2;

	private static final String LAST_UPDATED_DATE = "lastUpdatedDate";
	private static final String PRIMARY_CARE_MAX_LIMIT_ERROR_MSG = "Number of Request Exceeded:You can submit one primary care appointment request at a time. "
			+ "Another request may be submitted after the current primary care request is processed by the VA, or you may return to the previous screen, cancel the pending request, and submit a new one.";
	private static final String MENTAL_HEALTH_MAX_LIMIT_ERROR_MSG = "Number of Request Exceeded:You can only have two pending mental health appointment requests at a time. "
			+ "Another request may be submitted after one or more existing mental health appointment requests are processed by the VA, or you may return to the previous screen, cancel a pending request, and submit a new one.";
	private static final String FACILITY_NOT_FOUND_ERROR_MSG = "The requested facility was not found.  Please choose the desired facility using the autocomplete feature.";
	private static final String STATUS_NOT_FOUND_ERROR_MSG = "The appointment request is missing a valid status.";
	private static final String INVALID_STATUS_ERROR_MSG = "The requested appointment request status is invalid.";
    private static final String APPOINTMENT_REQUEST_MSG_NOT_ALLOWED_FOR_CANCELLED_APPOINTMENT = "Appointment request messages are not allowed for cancelled appointment requests.";
    private static final String APPOINTMENT_REQUEST_MSG_NOT_ALLOWED_AFTER_APPOINTMENT_DATE_PASSED = "Appointment request messages are not allowed after appointment date has passed.";
    private static final String APPOINTMENT_REQUEST_MSG_NOT_ALLOWED_UNTIL_REPLY = "New appointment request message not allowed until reply received.";
    private static final String APPOINTMENT_REQUEST_MSG_LIMIT_REACHED = "Maximum allowed number of messages for this appointment request reached.";
    private static final String VAR_SYSTEM_IDENTIFIER = "var";

	public VARAppointmentRequests getPatientAppointmentRequests(PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter) {
        VARAppointmentRequests appointmentRequests = readAppointmentRequestsFromRouterResponseAndSortDescendingByLastUpdatedDate(appointmentRequestDataLayer.fetchAppointmentRequestsByPatient(patientIdentifier.getUniqueId(),dateFilter), patientIdentifier);
		return appointmentRequests;
	}

	public Facilities fetchMostRecentFacilities(PatientIdentifier patientIdentifier, ScopeFilter scopeFilter) {
        VARAppointmentRequests patientAppointmentRequests = getPatientAppointmentRequests(patientIdentifier, DateFilterFactory.createEmptyFilter(), scopeFilter);
		patientAppointmentRequests.sortDescending("createdDate");

		Facilities facilities = new Facilities();

		for (VARAppointmentRequest appointmentRequest : patientAppointmentRequests) {
			Facility facility = appointmentRequest.getFacility();

			if (facilities.size() < 3) {
				if (!facilities.contains(facility)) {
					facilities.add(facility);
				}
			} else {
				break;
			}
		}

		return facilities;
	}

	public VARAppointmentRequests getAppointmentRequests(AppointmentRequestFilter filter) {
        VARAppointmentRequests appointmentRequests = readAppointmentRequestsFromRouterResponseAndSortAscendingByLastUpdatedDate(appointmentRequestDataLayer.fetchAppointmentRequests(filter));
        VARAppointmentRequests filteredAppointmentRequests = excludeAppointmentRequestsCancelledByVeteranBeforeBeingBooked(appointmentRequests);

		return filteredAppointmentRequests;
	}

    public VARAppointmentRequests getAppointmentRequestsNoSort(AppointmentRequestFilter filter) {
        VARAppointmentRequests appointmentRequests = appointmentRequestDataLayer.fetchAppointmentRequests(filter);
        VARAppointmentRequests filteredAppointmentRequests = excludeAppointmentRequestsCancelledByVeteranBeforeBeingBooked(appointmentRequests);

        return filteredAppointmentRequests;
    }


	private VARAppointmentRequests excludeAppointmentRequestsCancelledByVeteranBeforeBeingBooked(VARAppointmentRequests appointmentRequests) {
        VARAppointmentRequests filteredAppointmentRequests = new VARAppointmentRequests();

		for (VARAppointmentRequest appointmentRequest : appointmentRequests) {
			if (isCancelledAppointmentRequest(appointmentRequest)) {
				if (doesAppointmentRequestContainDetailCode(appointmentRequest)) {
					filteredAppointmentRequests.add(appointmentRequest);
				}
			} else {
				filteredAppointmentRequests.add(appointmentRequest);
			}
		}

		return filteredAppointmentRequests;
	}

	private boolean doesAppointmentRequestContainDetailCode(VARAppointmentRequest appointmentRequest) {
		List<VARAppointmentRequestDetailCode> arDetailCodes = appointmentRequest.getAppointmentRequestDetailCode();
		if (NullChecker.isNotNullish(arDetailCodes)) {
			return true;
		}
		return false;
	}

	private boolean isCancelledAppointmentRequest(VARAppointmentRequest appointmentRequest) {
		if (AppointmentRequestStatus.CANCELLED.getName().equalsIgnoreCase(appointmentRequest.getStatus())) {
			return true;
		}
		return false;
	}

	private VARAppointmentRequests readAppointmentRequestsFromRouterResponseAndSortDescendingByLastUpdatedDate(VARAppointmentRequests appointmentRequests, PatientIdentifier patientIdentifier) {
		//AppointmentRequests appointmentRequests = PatientDataResponseReader.<AppointmentRequests, AppointmentRequest> fromResponse(responseMessage).getDataListNoNull(AppointmentRequests.class);
		//appointmentRequests.sortDescending(LAST_UPDATED_DATE);
        for (VARAppointmentRequest appointmentRequest : appointmentRequests){
            appointmentRequest.setPatientIdentifier(new PatientIdentifier(patientIdentifier.getAssigningAuthority(), appointmentRequest.getPatientIdentifier().getUniqueId()));
            appointmentRequest.setDataIdentifier(new DataIdentifier(VAR_SYSTEM_IDENTIFIER, appointmentRequest.getDataIdentifier().getUniqueId()));
        }
		return appointmentRequests;
	}

	private VARAppointmentRequests readAppointmentRequestsFromRouterResponseAndSortAscendingByLastUpdatedDate(VARAppointmentRequests appointmentRequests) {
		appointmentRequests.sortAscending(LAST_UPDATED_DATE);
		return appointmentRequests;
	}

	public VARAppointmentRequest getPatientAppointmentRequest(PatientIdentifier patientIdentifier, DataIdentifier dataIdentifier) {
        VARAppointmentRequest appointmentRequest = appointmentRequestDataLayer.fetchAppointmentRequest(patientIdentifier.getUniqueId(), dataIdentifier.getUniqueId());
        appointmentRequest.setPatientIdentifier(new PatientIdentifier(patientIdentifier.getAssigningAuthority(), appointmentRequest.getPatientIdentifier().getUniqueId()));
        appointmentRequest.setDataIdentifier(new DataIdentifier(VAR_SYSTEM_IDENTIFIER, appointmentRequest.getDataIdentifier().getUniqueId()));
    
		return appointmentRequest;
	}

	public VARAppointmentRequestMessages fetchAppointmentRequestMessages(PatientIdentifier patientIdentifier, DataIdentifier dataIdentifier) {
        VARAppointmentRequestMessages messages = appointmentRequestDataLayer.fetchAppointmentRequestMessages(patientIdentifier.getUniqueId(),dataIdentifier.getUniqueId());

        for(VARAppointmentRequestMessage appointmentRequestMessage : messages){
            appointmentRequestMessage.setPatientIdentifier(patientIdentifier);
            appointmentRequestMessage.setDataIdentifier(dataIdentifier);
        }
		messages = readAppointmentRequestMessagesFromRouterResponseAndSortDescendingMessageDateTime(messages);
		return messages;
	}

	private VARAppointmentRequestMessages readAppointmentRequestMessagesFromRouterResponseAndSortDescendingMessageDateTime(VARAppointmentRequestMessages messages) {
		messages.sortDescending("messageDateTime");
		return messages;
	}

	public VARAppointmentRequest saveAppointmentRequest(List<String> pilotSites, VARAppointmentRequest appointmentRequest, String haMode, boolean fullValidation, ScopeFilter scopeFilter) {
		Patient patient = null;
		if (haMode.equalsIgnoreCase(RunningMode.VETERAN.getName())) {
			patient = MhpUserFactory.createFromSecurityContext().getPatient();
		} else {
			patient = appointmentRequest.getPatient();
		}

		if (NullChecker.isNullish(patient.getPatientIdentifier().getUniqueId())) {
			patient.setPatientIdentifier(appointmentRequest.getPatientIdentifier());
		}
        
		patient.setTextMessagingAllowed(appointmentRequest.isTextMessagingAllowed());
		patient.setTextMessagingPhoneNumber(appointmentRequest.getTextMessagingPhoneNumber());

		appointmentRequest.setPatient(patient);
		if (appointmentRequest.getParentRequest() != null) {
			appointmentRequest.getParentRequest().setPatient(patient);
		}

		validateAppointmentRequest(pilotSites, appointmentRequest, fullValidation);

        addSurrogate(appointmentRequest);

        VARAppointmentRequest result = appointmentRequestDataLayer.saveAppointmentRequest(appointmentRequest);
        result.setPatientIdentifier(appointmentRequest.getPatientIdentifier()); 
        return result;
	}

	public VARAppointmentRequestMessage saveAppointmentRequestMessage(VARAppointmentRequestMessage appointmentRequestMessage) {

        VARAppointmentRequest associatedAppointmentRequest = getPatientAppointmentRequest(appointmentRequestMessage.getPatientIdentifier(), appointmentRequestMessage.getDataIdentifier());

        validateAppointmentRequestMessage(appointmentRequestMessage, associatedAppointmentRequest);

        addSurrogate(appointmentRequestMessage);
        boolean isProvider = MhpUserFactory.doesUserHasStaffRole();
        VARAppointmentRequestMessage result = appointmentRequestDataLayer.saveAppointmentRequestMessage(appointmentRequestMessage.getPatientId(), appointmentRequestMessage, isProvider);

		return result;
	}

    private void validateAppointmentRequestMessage(VARAppointmentRequestMessage appointmentRequestMessage, VARAppointmentRequest associatedAppointmentRequest) {

        AppointmentRequestMessageValidator validator = AppointmentRequestMessageValidatorFactory.createAppointmentRequestMessageValidator();

        if (validator != null) {
            ValidationResult<VARAppointmentRequestMessage> validationResult = validator.validate(appointmentRequestMessage);
            if (!validationResult.isValid()) {
                throw new InvalidAppointmentRequestMessageException(validationResult.getErrors());
            }
        }

        throwAnErrorIfAppointmentRequestIsCancelled(associatedAppointmentRequest, appointmentRequestMessage);
        throwAnErrorIfAppointmentRequestDateHasPassed(associatedAppointmentRequest, appointmentRequestMessage);
        //throwAnErrorIfNewMessageHasNotReceivedReply(associatedAppointmentRequest, appointmentRequestMessage);
        throwAnErrorIfMessageLimitReached(associatedAppointmentRequest, appointmentRequestMessage);
    }

    public VARAppointmentRequest updateAppointmentMessageFlag(String appointmentRequestId, PatientIdentifier patientIdentifier, DataIdentifier dataIdentifier){
		boolean isProvider = MhpUserFactory.doesUserHasStaffRole();
        VARAppointmentRequest appointmentRequest = appointmentRequestDataLayer.updateMessageFlag(patientIdentifier.getUniqueId(), appointmentRequestId, isProvider);
        MhpUserFactory.addSurrogateToPatientData(appointmentRequest);
		return appointmentRequest;
	}
	
    public void deleteAppointmentRequest(VARAppointmentRequest appointmentRequest, ScopeFilter scopeFilter) {
        addSurrogate(appointmentRequest);
		appointmentRequestDataLayer.deleteAppointmentRequest(appointmentRequest);
	}

	private void validateAppointmentRequest(List<String> pilotSites, VARAppointmentRequest appointmentRequest, boolean fullValidation) {
		if (fullValidation) {
			if (AppointmentRequestStatus.SUBMITTED.getName().equalsIgnoreCase(appointmentRequest.getStatus()) && !appointmentRequest.isSecondRequest()) {
				validateMaximumNumberOfAppointmentRequests(appointmentRequest);
			}
		}

		AppointmentRequestValidator validator = AppointmentRequestValidatorFactory.createAppointmentRequestValidator(EarliestTimeVeteranCanRequestAppointmentInHours, FarthestTimeVeteranCanRequestAppointmentInDays);

		if (validator != null) {
			ValidationResult<VARAppointmentRequest> validationResult = validator.validate(appointmentRequest, fullValidation);
			if (!validationResult.isValid()) {
				throw new InvalidAppointmentRequestException(validationResult.getErrors());
			}
		}
		//validateFacilityCode(pilotSites, appointmentRequest);
		validateStatus(appointmentRequest);
	}

	private void validateStatus(VARAppointmentRequest appointmentRequest) {
		String status = appointmentRequest.getStatus();

		if (NullChecker.isNotNullish(status)) {
			boolean found = false;
			AppointmentRequestStatus[] statuses = AppointmentRequestStatus.values();
			for (AppointmentRequestStatus statusEnum : statuses) {
				if (status.equals(statusEnum.getName())) {
					found = true;
					break;
				}
			}

			if (!found) {
				ValidationResult<VARAppointmentRequest> validationResult = createOpenAppointmentRequestValidationError(appointmentRequest, INVALID_STATUS_ERROR_MSG);
				throw new InvalidAppointmentRequestException(validationResult.getErrors());
			}
		} else {
			ValidationResult<VARAppointmentRequest> validationResult = createOpenAppointmentRequestValidationError(appointmentRequest, STATUS_NOT_FOUND_ERROR_MSG);
			throw new InvalidAppointmentRequestException(validationResult.getErrors());
		}
	}

	/*private void validateFacilityCode(List<String> pilotSites, VARAppointmentRequest appointmentRequest) {
		boolean notFound = false;
		try {
			String facilityCode = appointmentRequest.getFacility().getFacilityCode();
			Facility facility = facilityDataService.fetchFacilityByCode(pilotSites, facilityCode);

			if (facility == null) {
				notFound = true;
			}
		} catch (WebApplicationException e) {
			notFound = true;
		}

		if (notFound) {
			ValidationResult<VARAppointmentRequest> validationResult = createValidationError(appointmentRequest, "facilityName", FACILITY_NOT_FOUND_ERROR_MSG);
			throw new InvalidAppointmentRequestException(validationResult.getErrors());
		}
	}*/

	private void validateMaximumNumberOfAppointmentRequests(VARAppointmentRequest appointmentRequest) {
		if (AppointmentRequestType.PRIMARY_CARE.getName().equals(appointmentRequest.getAppointmentType())) {
			throwAnErrorIfPrimaryCareAppointmentRequestsExist(appointmentRequest);
		} else if (AppointmentRequestType.MENTAL_HEALTH.getName().equals(appointmentRequest.getAppointmentType())) {
			throwAnErrorIfMentalHealthAppointmentRequestsExist(appointmentRequest);
		}
	}
	
	private ValidationResult<VARAppointmentRequest> createValidationError(VARAppointmentRequest appointmentRequest, String fieldName, String errorMessage) {
		ValidationResult<VARAppointmentRequest> validationResult = new ValidationResult<VARAppointmentRequest>();
		validationResult.setRequestObject(appointmentRequest);

		ValidationError error = new ValidationError();
		error.setErrorMessage(errorMessage);
		error.setFieldName(fieldName);

		validationResult.addIfNotNull(error);
		return validationResult;
	}

	private void throwAnErrorIfMentalHealthAppointmentRequestsExist(VARAppointmentRequest appointmentRequest) {
		if (areThereMoreThanOneSubmittedMentalHealthAppointmentRequests(appointmentRequest)) {
			ValidationResult<VARAppointmentRequest> validationResult = createOpenAppointmentRequestValidationError(appointmentRequest, MENTAL_HEALTH_MAX_LIMIT_ERROR_MSG);
			throw new InvalidAppointmentRequestException(validationResult.getErrors());
		}
	}

	private boolean areThereMoreThanOneSubmittedMentalHealthAppointmentRequests(VARAppointmentRequest submittedAppointmentRequest) {
		DateFilter defaultDateFilter = createDateFilterWithDefaultDateRange();

        VARAppointmentRequests patientAppointmentRequests = getPatientAppointmentRequests(submittedAppointmentRequest.getPatientIdentifier(), defaultDateFilter, ScopeFilter.getInstanceForLongitudinalScope());

		int recordCount = 0;
		for (VARAppointmentRequest appointmentRequest : patientAppointmentRequests) {
			if (appointmentRequest.getAppointmentType().equals(AppointmentRequestType.MENTAL_HEALTH.getName()) && appointmentRequest.getStatus().equals(AppointmentRequestStatus.SUBMITTED.getName())) {
				if (++recordCount > 1) {
					return true;
				}
			}
		}

		return false;
	}

	private void throwAnErrorIfPrimaryCareAppointmentRequestsExist(VARAppointmentRequest appointmentRequest) {
		if (areThereAnySubmittedPrimaryCareAppointmentRequests(appointmentRequest)) {
			ValidationResult<VARAppointmentRequest> validationResult = createOpenAppointmentRequestValidationError(appointmentRequest, PRIMARY_CARE_MAX_LIMIT_ERROR_MSG);
			throw new InvalidAppointmentRequestException(validationResult.getErrors());
		}
	}

	private ValidationResult<VARAppointmentRequest> createOpenAppointmentRequestValidationError(VARAppointmentRequest appointmentRequest, String errorMessage) {
		return createValidationError(appointmentRequest, "error", errorMessage);
	}

    private void throwAnErrorIfAppointmentRequestDateHasPassed(VARAppointmentRequest appointmentRequest, VARAppointmentRequestMessage appointmentRequestMessage) {
        if (!hasAppointmentDatePassed(appointmentRequest.getAppointmentDate())){
            ValidationResult<VARAppointmentRequestMessage> validationResult = createOpenAppointmentRequestMessageValidationError(appointmentRequestMessage, APPOINTMENT_REQUEST_MSG_NOT_ALLOWED_AFTER_APPOINTMENT_DATE_PASSED);
            throw new InvalidAppointmentRequestMessageException(validationResult.getErrors());
        }
    }
    private boolean hasAppointmentDatePassed(String appointmentRequestDate){

        if(NullChecker.isNotNullish(appointmentRequestDate)){
            Date today = DateHelper.beginOfDate(DateHelper.getToday());
            Date appointmentDate = DateHelper.beginOfDate(DateHelper.parseDate(appointmentRequestDate));
            if(today.after(appointmentDate))
                return false;
        }

        return true;
    }

    private void throwAnErrorIfAppointmentRequestIsCancelled(VARAppointmentRequest appointmentRequest, VARAppointmentRequestMessage appointmentRequestMessage) {
        if (appointmentRequest.getStatus().equals("Cancelled")){
            ValidationResult<VARAppointmentRequestMessage> validationResult = createOpenAppointmentRequestMessageValidationError(appointmentRequestMessage, APPOINTMENT_REQUEST_MSG_NOT_ALLOWED_FOR_CANCELLED_APPOINTMENT);
            throw new InvalidAppointmentRequestMessageException(validationResult.getErrors());
        }
    }

    private ValidationResult<VARAppointmentRequestMessage> createOpenAppointmentRequestMessageValidationError(VARAppointmentRequestMessage appointmentRequestMessage, String errorMessage) {
        ValidationResult<VARAppointmentRequestMessage> validationResult = new ValidationResult<VARAppointmentRequestMessage>();
        validationResult.setRequestObject(appointmentRequestMessage);

        ValidationError error = new ValidationError();
        error.setErrorMessage(errorMessage);
        error.setFieldName("error");

        validationResult.addIfNotNull(error);
        return validationResult;
    }


    private void throwAnErrorIfNewMessageHasNotReceivedReply(VARAppointmentRequest appointmentRequest, VARAppointmentRequestMessage appointmentRequestMessage){

        VARAppointmentRequestMessages currentMessagesForAppointment =  fetchAppointmentRequestMessages(appointmentRequest.getPatientIdentifier(), appointmentRequest.getDataIdentifier());

       if(!currentMessagesForAppointment.isEmpty()){
           VARAppointmentRequestMessage latestMessage = currentMessagesForAppointment.get(0);
           if(latestMessage.getSenderId().equals(appointmentRequestMessage.getSenderId())){
               ValidationResult<VARAppointmentRequestMessage> validationResult = createOpenAppointmentRequestMessageValidationError(appointmentRequestMessage, APPOINTMENT_REQUEST_MSG_NOT_ALLOWED_UNTIL_REPLY);
               throw new InvalidAppointmentRequestMessageException(validationResult.getErrors());
           }
       }


    }


    private void throwAnErrorIfMessageLimitReached(VARAppointmentRequest appointmentRequest, VARAppointmentRequestMessage appointmentRequestMessage){

        VARAppointmentRequestMessages currentMessagesForAppointment =  fetchAppointmentRequestMessages(appointmentRequest.getPatientIdentifier(), appointmentRequest.getDataIdentifier());
        int numberOfMessagesAlreadySentBySender = 0;
        for(VARAppointmentRequestMessage aptReqMsg : currentMessagesForAppointment){
            if(appointmentRequestMessage.getSenderId().equals(aptReqMsg.getSenderId())){
                numberOfMessagesAlreadySentBySender ++;
            }
        }

        if(numberOfMessagesAlreadySentBySender >= messageLimitPerSender){
            ValidationResult<VARAppointmentRequestMessage> validationResult = createOpenAppointmentRequestMessageValidationError(appointmentRequestMessage, APPOINTMENT_REQUEST_MSG_LIMIT_REACHED);
            throw new InvalidAppointmentRequestMessageException(validationResult.getErrors());
        }

    }


    private boolean areThereAnySubmittedPrimaryCareAppointmentRequests(VARAppointmentRequest submittedAppointmentRequest) {
		DateFilter defaultDateFilter = createDateFilterWithDefaultDateRange();

		VARAppointmentRequests patientAppointmentRequests = getPatientAppointmentRequests(submittedAppointmentRequest.getPatientIdentifier(), defaultDateFilter, ScopeFilter.getInstanceForLongitudinalScope());

		for (VARAppointmentRequest appointmentRequest : patientAppointmentRequests) {
			if (appointmentRequest.getAppointmentType().equals(AppointmentRequestType.PRIMARY_CARE.getName()) && appointmentRequest.getStatus().equals(AppointmentRequestStatus.SUBMITTED.getName())) {
				return true;
			}
		}

		return false;
	}

	private DateFilter createDateFilterWithDefaultDateRange() {
		Date endDate = DateHelper.getToday();
		Date startDate = DateHelper.beginOfDate(DateHelper.minusDays(endDate, FarthestTimeVeteranCanRequestAppointmentInDays));
		return DateFilterFactory.createFilterFromDate(startDate, endDate);
	}
	
	//TODO:Confirm if this is scheduling manager method
	/*public AppointmentRequest updateProviderSeeAppointmentRequestFlag(String appointmentRequestId, PatientIdentifier patientIdentifier, DataIdentifier dataIdentifier){
		RequestMessage request = UpdateProviderSeenAppointmentRequestFlagRequestBuilder.forUpdateProviderSeenAppointmentRequestFlag().forDomain(Domain.appointmentRequest).forPatientIdentifier(patientIdentifier).forDataIdentifier(dataIdentifier).forScopeFilter(ScopeFilter.getInstanceForLongitudinalScope()).forAppointmentRequestId(appointmentRequestId).build();
		ResponseMessage responseMessage = router.execute(request);
		return PatientDataResponseReader.<AppointmentRequests, AppointmentRequest>fromResponse(responseMessage).getDataItem();
	}*/
	
	private void addSurrogate(PatientData patientData) {
        MhpUserFactory.addSurrogateToPatientData(patientData);
	}
}
