define([
	'Global',
	'BookedAppointments',
	'AvailableApptSlots',
    'Moment'
],
	function(Global, BookedAppointments, AvailableApptSlots) {
		'use strict';
		var CalendarData = Backbone.Model.extend({
			
			defaults: {
				"size": 0,
				"clinicId": "",
				"clinicName": "",
				"appointmentLength": "0",
				"clinicDisplayStartTime": "0",
                "defaultDate": moment().format("MM/DD/YYYY"),
				"displayIncrements": "0",
				"stopCode": "0",
				"creditCode": "0",
				"specialty": "0",
				"askForCheckIn": false,
				"maxOverbooksPerDay": "0",
				"hasUserAccessToClinic": false,
				"appointmentTimeSlot": new Backbone.Collection({comparator: "start"}),
                "alreadyFetched": false
			},
            initialize: function () {
                this.today = moment();
            },
			parse: function(response){
                var slots = response.appointmentTimeSlot,
	                availableSlots = new AvailableApptSlots(slots, {parse: true});

				response.appointmentTimeSlot = availableSlots;
				response.numAvailableSlots = availableSlots.length;

				return response;

			},
			getSlotsForDay: function(date){
                date = moment(date);

                var timeSlots = this.get('appointmentTimeSlot'),
                    daySlots = new AvailableApptSlots(this.filterSlots(timeSlots, date));

                return daySlots.toJSON();
			},
            filterSlots: function (slots, date) {
                var that = this;

                return slots.filter(function (model) {
                    var slotTime = moment(model.get('start')),
                        slotDate = that.clearTime(moment(model.get('start')));

                    return slotTime > that.today &&
                            slotDate.isSame(date) &&
                            (!that.isSlotHidden(model.toJSON()) || model.get('appointmentId'));
                });
            },
            isSlotHidden: function (slot) {
                var hideSlot = false,
                    status = slot.bookingStatus + "",
                    availability = slot.availability,
                    statusAsInt = parseInt(status);

                if (!availability || statusAsInt < 1 || isNaN(statusAsInt)) {
                    hideSlot = true;
                }

                return hideSlot;
            },
            addSlots: function(patientAppts) {
                var that = this,
                    timeSlots = this.get('appointmentTimeSlot'),
                    removeSlots = [];

                _.each(patientAppts, function(appt) {
                    for (var i = 0; i < timeSlots.length; i++) {
                        var slot = timeSlots.at(i);

                        if (that.slotCollidesWithAppt(slot.toJSON(), appt)) {
                            removeSlots.push(slot);
                        }
                    }
                });

                timeSlots.remove(removeSlots);
                timeSlots.add(patientAppts);

                this.set({"appointmentTimeSlot": timeSlots});
            },
            getSlots: function() {
                return this.get('appointmentTimeSlot').toJSON();
            },
            slotCollidesWithAppt: function(slot, appt) {
                var slotStart = new moment(slot.start),
                    slotEnd = new moment(slot.end),
	                slotDay = slotStart.format("MMDD"),
                    apptStart = new moment(appt.start),
                    apptEnd = new moment(appt.end),
	                apptDay = apptStart.format("MMDD"),
                    collides = false;

                if (appt.appointmentDesignation === "PRIMARY_CARE" && apptDay === slotDay) {
	                collides = true;
                } else if ((slotStart <= apptStart && apptStart < slotEnd) ||
                    (slotStart < apptEnd && apptEnd <= slotEnd) ||
                    (slotStart >= apptStart && slotEnd <= apptEnd)) {
                    collides = true;
                }

                return collides;
            },
            updateFacility: function (facility) {
                this.set({
	                "facilityCode": facility.get('facilityId'),
                    "facilityName": facility.get('facilityName'),
	                "providerName": facility.get('providerName')
                });
            },
            clearTime: function (date) {
                date = moment(date);

                return date.hours(0).minutes(0).seconds(0).milliseconds(0);
            }
		});

		return CalendarData;
	}
);