define(['Global', 'Vent', 'AppointmentRequest', 'AppointmentRequestsListView'],
	function(Global, vent, AppointmentRequest, AppointmentRequestsListView){
		"use strict";

		describe("Appointment Requests List View", function(){
			var model = new Backbone.Model({
					emailAddress: "",
					emailAllowed: false,
					notificationFrequency: "Never"
				}),
				collection = Backbone.Collection.extend({
					model: AppointmentRequest,
					countUnreadMessages: function () {
						return 0;
					}
				}),
				view = new AppointmentRequestsListView({
					model: model,
					collection: new collection()
				}),
				collectionData = [
					{
						lastAccessDate: "",
						status: "Submitted",
						facility: {
							name: "TEST VAMC 3"
						},
						appointmentType: "Primary Care",
						hasVeteranNewMessage: ""
					},
					{
						lastAccessDate: "",
						status: "Submitted",
						facility: {
							name: "TEST VAMC 3"
						},
						appointmentType: "Mental Health",
						hasVeteranNewMessage: ""
					},
					{
						lastAccessDate: "",
						status: "Booked",
						facility: {
							name: "TEST VAMC 3"
						},
						appointmentType: "Primary Care",
						hasVeteranNewMessage: ""
					},
					{
						lastAccessDate: "",
						status: "Not Booked",
						facility: {
							name: "TEST VAMC 3"
						},
						appointmentType: "Primary Care",
						hasVeteranNewMessage: ""
					},
					{
						lastAccessDate: "",
						status: "Cancelled",
						facility: {
							name: "TEST VAMC 3"
						},
						appointmentType: "Primary Care",
						hasVeteranNewMessage: ""
					}
				];

			view.render();

			beforeEach(function () {
				spyOn(vent, "trigger");
			});

			it("sets the seen request list instructions", function(){
				var getSetSeenInstructions = Global.getUtilities().getSetSeenInstructions;

				getSetSeenInstructions({
					userId: "testpatient",
					action: "set",
					setValue: false
				});

				spyOn(Global.getUtilities(), "getSetSeenInstructions").andCallThrough();

				view.setSeenRequestListInstructions();

				expect(Global.getUtilities().getSetSeenInstructions).toHaveBeenCalled();
				expect(getSetSeenInstructions({userId: "testpatient", action: "get"})).toBeTruthy();
			});

			it("updates the instructional header based on the length of the collection", function () {
				view.updateInstructionalHeader();

				expect(vent.trigger).toHaveBeenCalledWith("update:instructional-header", "You have not requested any appointments");

				view.collection.set(collectionData);

				view.updateInstructionalHeader();

				expect(vent.trigger).toHaveBeenCalledWith("update:instructional-header", "Read notifications or update your notification settings");
			});

			it("has a function to add the status filter dropdown", function () {
				spyOn($.fn, "trigger");

				view.render();

				$.fn.trigger.reset();

				localStorage.setItem(view.statusFilterKey, "");

				expect(view.ui.list.children().length).toEqual(5);

				view.addFilterDropDown([], 0);

				expect(view.ui.statusFilter.children().length).toBeFalsy();

				view.addFilterDropDown({
					"Submitted": 2,
					"Booked": 1,
					"Not Booked": 1,
					"Cancelled": 1
				}, 5);

				var filterOptions = view.ui.statusFilter.find('select').children();

				expect(filterOptions.length).toEqual(5);
				expect(filterOptions.eq(0).text()).toEqual("All (5)");
				expect(filterOptions.eq(1).text()).toEqual("Submitted (2)");
				expect(filterOptions.eq(2).text()).toEqual("Booked (1)");
				expect(filterOptions.eq(3).text()).toEqual("Not Booked (1)");
				expect(filterOptions.eq(4).text()).toEqual("Cancelled (1)");

				$.fn.trigger.reset();

				expect($.fn.trigger).not.toHaveBeenCalled();

				localStorage.setItem(view.statusFilterKey, "All");

				view.addFilterDropDown({
					"Submitted": 2,
					"Booked": 1,
					"Not Booked": 1,
					"Cancelled": 1
				}, 5);

				expect($.fn.trigger).toHaveBeenCalledWith("change");

			});

			it("has a function to filter the list on a given status", function () {
				view.filterListOn("blah");
				expect(view.ui.list.children().is(':visible')).toBeFalsy();

				view.filterListOn("Submitted");
				expect(view.ui.list.find("li:not(.hidden)").length).toEqual(2);
			});

			it("has a function to filter the list", function () {
				spyOn(view, "filterListOn").andCallThrough();

				var value = "All",
					e = {
						currentTarget: {
							querySelector: function (selector) {return {value: value}}
						}
					};

				view.filterList(e);
				expect(view.filterListOn).not.toHaveBeenCalled();
				expect(view.ui.list.find("li:not(.hidden)").length).toEqual(5);

				value = "Submitted";

				view.filterList(e);
				expect(view.filterListOn).toHaveBeenCalledWith("Submitted");
				expect(view.ui.list.find("li:not(.hidden)").length).toEqual(2);
			});

			it("can show all the list items", function () {
				view.showAllListItems();
				expect(view.ui.list.find("li.hidden").length).toEqual(0);
			});

			it("can hide all the list items", function () {
				view.hideAllListItems();
				expect(view.ui.list.find("li.hidden").length).toEqual(5);
			});

			it("has a function to get the statuses of the list for the filter dropdown", function () {
				spyOn(view, "addFilterDropDown");

				view.getStatuses();
				expect(view.addFilterDropDown).toHaveBeenCalledWith({
					"Submitted": 2,
					"Booked": 1,
					"Not Booked": 1,
					"Request Cancelled": 1
				}, 5);

				view.ui.list.empty();

				view.addFilterDropDown.reset();

				view.getStatuses();
				expect(view.addFilterDropDown).not.toHaveBeenCalled();
			});

			it("has a function to have one of the two sections open by default", function () {
				view.openCloseSections();
				expect(view.ui.requests.hasClass('no-jqm-collapsible-open')).toBeTruthy();
				expect(view.ui.notifications.hasClass('no-jqm-collapsible-open')).toBeTruthy();

				view.openSection = "requests";
				view.openCloseSections();
				expect(view.ui.requests.hasClass('no-jqm-collapsible-open')).toBeTruthy();
				expect(view.ui.notifications.hasClass('no-jqm-collapsible-open')).toBeFalsy();

				view.ui.notifications.addClass('no-jqm-collapsible-open');

				view.openSection = "settings";
				view.openCloseSections();
				expect(view.ui.requests.hasClass('no-jqm-collapsible-open')).toBeFalsy();
				expect(view.ui.notifications.hasClass('no-jqm-collapsible-open')).toBeTruthy();
			});

			it("allows the user to go back to the main landing page", function () {
				view.goBack();
				expect(vent.trigger).toHaveBeenCalledWith('show:landing');
			});

		});
	}
);