define(['Global', 'Vent', 'LandingPageView', 'LandingPage', 'User'],
	function(Global, vent, LandingPageView, LandingPage, User){
		"use strict";

		LandingPage.initialize = function(){};


		var AppointmentRequests = Backbone.Collection.extend({
				countUnreadMessages: function () {
					return _.filter(this.models, function(model){ return model.get("hasVeteranNewMessage") } ).length;
				}
			}),
			items	= {
				booked: {
					"id":"schedule-list",
					"href": "#schedule-list",
					"text": "Booked VA Appointments",
					"description": "View a list of your appointments for the next 120 days",
					"priority": 10
				},
				requests: {
					"id":"requests",
					"href": "#requests",
					"text": "Request Appointment",
					"description": "Request Primary Care or Mental Health Appointments, and message with scheduling clerks",
					"priority": 20,
					"checkMessages": true
				},
				scheduleAppt: {
					"id":"direct-scheduling",
					"href": "#direct-scheduling",
					"text": "Manage My Appointments",
					"description": "Schedule or Cancel Primary Care Appointments from Clinic calendar",
					"priority": 50
				}
			},
			itemCollection 	= new LandingPage([items.booked, items.requests, items.scheduleAppt]),
			emptyCollection	= new Backbone.Collection(),
			landingPage 	= new LandingPageView({
									collection: itemCollection,
									appointments: emptyCollection,
									requests: new AppointmentRequests()
								});

		function resetItems(config, render){
			landingPage.close();
			$.extend(true, items, config);
			itemCollection.reset([ items.booked, items.requests, items.scheduleAppt ]);
			if( render ){
				landingPage.render();
			}
		}
		landingPage.render();

		describe("Landing Page", function(){
			var landingPageLinkHeaders = landingPage.$el.find('a h2');

			beforeEach(function () {
				spyOn(vent, "trigger");
			});

			it("has 3 sections", function(){
				expect(landingPageLinkHeaders.length).toEqual(3);
			});

			it("has items in order", function(){
				expect(landingPageLinkHeaders.eq(0).text().trim()).toMatch(/^Booked VA Appointments$/);
				expect(landingPageLinkHeaders.eq(1).text().trim()).toMatch(/^Request Appointment/);
				expect(landingPageLinkHeaders.eq(2).text().trim()).toMatch(/^Manage My Appointments$/);
			});

			it("has a priority on items ordering", function(){
				resetItems({booked:{priority:70}}, true);
				var landingPageLinkHeaders = landingPage.$el.find('a h2');
				expect(landingPageLinkHeaders.eq(0).text().trim()).toMatch(/^Request Appointment/);
				expect(landingPageLinkHeaders.eq(1).text().trim()).toMatch(/^Manage My Appointments$/);
				expect(landingPageLinkHeaders.eq(2).text().trim()).toMatch(/^Booked VA Appointments$/);
			});

			it("navigates correctly", function () {
				landingPage.showRequests();
				expect(vent.trigger).toHaveBeenCalledWith("show:request:listing");

				landingPage.showBookedAppointments();
				expect(vent.trigger).toHaveBeenCalledWith("show:direct-scheduling:view");

				landingPage.showBookingLandingPage();
				expect(vent.trigger).toHaveBeenCalledWith("show:bookingLandingPage");

				landingPage.showDSHowToEnroll();
				expect(vent.trigger).toHaveBeenCalledWith("show:direct-scheduling:how-to-enroll");
			});

			describe("Get Number of Unread Messages", function () {
				var utilities = Global.getUtilities(),
					userSeenInstruction;

				it("shows 1 unread message when there are no unread messages and the user has not seen the instructions", function () {
					utilities.getSetSeenInstructions({
						userId: User.id,
						action: "set",
						setValue: ""
					});

					userSeenInstruction = utilities.getSetSeenInstructions({
						userId: User.id,
						action: "get"
					});

					expect(userSeenInstruction).toBeFalsy();

					expect(landingPage.templateHelpers().getNumUnreadMessages()).toEqual(1);
				});

				it("shows the number of unread messages if the user has not seen the instructions but has requests", function () {
					userSeenInstruction = utilities.getSetSeenInstructions({
						userId: User.id,
						action: "get"
					});

					expect(userSeenInstruction).toBeFalsy();

					landingPage.requests.set([
						{},
						{},
						{},
						{}
					]);

					expect(landingPage.templateHelpers().getNumUnreadMessages()).toEqual(0);

					landingPage.requests.set([
						{
							hasVeteranNewMessage: true
						},
						{},
						{},
						{}
					]);

					expect(landingPage.templateHelpers().getNumUnreadMessages()).toEqual(1);
				});

				it("always shows the number of unread messages if the user has seen the instructions", function () {
					utilities.getSetSeenInstructions({
						userId: User.id,
						action: "set",
						setValue: true
					});

					userSeenInstruction = utilities.getSetSeenInstructions({
						userId: User.id,
						action: "get"
					});

					expect(userSeenInstruction).toBeTruthy();

					landingPage.requests.reset();

					expect(landingPage.templateHelpers().getNumUnreadMessages()).toEqual(0);

					landingPage.requests.set([
						{},
						{},
						{},
						{}
					]);

					expect(landingPage.templateHelpers().getNumUnreadMessages()).toEqual(0);

					landingPage.requests.set([
						{
							hasVeteranNewMessage: true
						},
						{},
						{},
						{}
					]);

					expect(landingPage.templateHelpers().getNumUnreadMessages()).toEqual(1);
				});
			});

		});
	}
);