define(['Global', 'NotificationSettingsView'],
	function(Global, NotificationSettingsView){
		"use strict";

		describe("Notifications Settings View", function(){
			var model = new Backbone.Model({
					emailAddress: "",
					emailAllowed: false,
					notificationFrequency: "Never"
				}),
				view = new NotificationSettingsView({
					model: model
				});

			view.render();

			it("has a function to set the notification preferences", function(){
				view.setNotificationPreferencesFields();

				expect(view.ui.emailUpdates[0].checked).toBeFalsy();
				expect(view.ui.emailSection.hasClass('hidden')).toBeTruthy();

				view.model.set({
					emailAllowed: true
				});

				view.setNotificationPreferencesFields();

				expect(view.ui.emailUpdates[0].checked).toBeTruthy();
				expect(view.ui.emailSection.hasClass('hidden')).toBeFalsy();

				view.model.set({
					emailAllowed: false
				});

				view.setNotificationPreferencesFields();
			});

			it("has a function to show or hide the email/frequency section", function () {
				view.ui.emailUpdates[0].checked = true;

				view.showHideEmailFrequency();

				expect(view.ui.emailSection.hasClass('hidden')).toBeFalsy();

				view.ui.emailUpdates[0].checked = false;

				view.showHideEmailFrequency();

				expect(view.ui.emailSection.hasClass('hidden')).toBeTruthy();
			});

			it("has a function to validate the email address entered by the user", function () {
				view.ui.email.val("");

				expect(view.validateEmail()).toBeFalsy();

				view.ui.email.val("test");

				expect(view.validateEmail()).toBeFalsy();

				view.ui.email.val("test@test.comtest@test.com");

				expect(view.validateEmail()).toBeFalsy();

				view.ui.email.val("test@test.com");

				expect(view.validateEmail()).toBeTruthy();

				view.ui.email.val("");
			});

			it("has a function to enable/disable the save preferences button", function () {
				expect(JSON.stringify(view.preferences.toJSON())).toEqual(JSON.stringify(view.model.toJSON()));

				view.enableSaveButton();

				expect(view.ui.savePreferences[0].disabled).toBeTruthy();

				view.ui.email.val("test@test.com");

				view.enableSaveButton();

				expect(view.ui.savePreferences[0].disabled).toBeTruthy();

				view.preferences.set({
					emailAllowed: true
				});

				view.enableSaveButton();

				expect(view.ui.savePreferences[0].disabled).toBeFalsy();

				view.model = view.preferences.clone();

				view.enableSaveButton();

				expect(view.ui.savePreferences[0].disabled).toBeTruthy();
			});

			it("has a function to update the selections in the view", function () {
				spyOn(view, "enableSaveButton");

				view.ui.email.val("");
				view.ui.emailUpdates[0].checked = false;
				view.ui.success.removeClass('hidden');

				view.updateSelections();

				expect(view.enableSaveButton).toHaveBeenCalled();
				expect(view.ui.success.hasClass('hidden')).toBeTruthy();

				expect(view.preferences.get("emailAddress")).toEqual("");
				expect(view.preferences.get("emailAllowed")).toEqual(false);
				expect(view.preferences.get("notificationFrequency")).toEqual("Never");

				view.ui.email.val("test@test.com");

				view.updateSelections();

				expect(view.preferences.get("emailAddress")).toEqual("");
				expect(view.preferences.get("emailAllowed")).toEqual(false);
				expect(view.preferences.get("notificationFrequency")).toEqual("Never");

				view.ui.emailUpdates[0].checked = true;

				view.updateSelections();

				expect(view.preferences.get("emailAddress")).toEqual("test@test.com");
				expect(view.preferences.get("emailAllowed")).toEqual(true);
				expect(view.preferences.get("notificationFrequency")).toEqual("Each new message");
			});

			it("updates the preferences", function () {
				var save = Backbone.Model.prototype.save;

				spyOn(Backbone.Model.prototype, "save").andCallFake(function (data, config){
					if ($.isFunction(config.error)) {
						config.error();
					}
				});

				view.model.set({
					emailAddress: "",
					emailAllowed: false,
					notificationFrequency: "Never"
				});

				view.ui.success.addClass("hidden");

				view.updatePreferences();

				expect(JSON.stringify(view.model.toJSON())).toEqual(JSON.stringify(view.preferences.toJSON()));

				expect(view.ui.successText.text()).toEqual("There was an error saving your preferences, please try again later.");
				expect(view.ui.successIcon.hasClass("icon-check")).toBeFalsy();
				expect(view.ui.successIcon.hasClass("icon-alert-2")).toBeTruthy();
				expect(view.ui.success.hasClass("hidden")).toBeFalsy();

				Backbone.Model.prototype.save = save;

				spyOn(Backbone.Model.prototype, "save").andCallFake(function (data, config) {
					if ($.isFunction(config.success)) {
						config.success();
					}
				});

				view.updatePreferences();

				expect(view.ui.successText.text()).toEqual("Preferences Saved!");
				expect(view.ui.successIcon.hasClass("icon-check")).toBeTruthy();
				expect(view.ui.successIcon.hasClass("icon-alert-2")).toBeFalsy();
				expect(view.ui.success.hasClass("hidden")).toBeFalsy();
			});

		});
	}
);