define(['Global', 'Vent', 'DSVeteranEmailView'],
	function(Global, vent, DSVeteranEmailView){
		'use strict';

		describe("Email Book/Cancel Confirmation Form", function(){
			var view = new DSVeteranEmailView({model: new Backbone.Model({
					start: moment("09/24/2014 10:00", "MM/DD/YYYY HH:mm"),
					facilityName: "TEST VAMC 3",
					clinic: "Primary Care",
					bookingNotes: "Notes",
					providerName: "",
					path: "book"
				})});

			view.render();

			beforeEach(function () {
				spyOn(vent, "trigger");
			});

			it("can navigate back to the landing page", function(){
				view.showLanding();
				expect(vent.trigger).toHaveBeenCalledWith('show:landing');
			});

			it("can show the confirmation that the email was sent", function () {
				view.ui.$email.val("test@test.com");
				view.showConfirmation();

				var args = vent.trigger.mostRecentCall.args,
					event = args[0],
					model = args[1],
					path = args[2];

				expect(event).toEqual('show:email-confirmation');
				expect(model.get('emailAddress')).toEqual('test@test.com');
				expect(path).toEqual('book');
			});

			it("has a template helper for getting the appointment date and time", function () {
				expect(view.templateHelpers().getAppointmentDateTime()).toEqual("Wednesday, September 24, 2014 @ 10:00");
			});

			it("has a template helper for getting facility name and clinic", function () {
				expect(view.templateHelpers().getFacilityClinic()).toEqual("TEST VAMC 3, Primary Care");

				view.model.set({
					clinic: {
						name: "General Medicine"
					},
					facility: {
						facilityName: "DC VAMC"
					},
					facilityName: ""
				});

				expect(view.templateHelpers().getFacilityClinic()).toEqual("DC VAMC, General Medicine");
			});

			it("has a template helper for getting booking notes", function () {
				expect(view.templateHelpers().getBookingNotes()).toEqual("Notes");

				view.model.set({
					bookingNotes: "",
					notes: "Some Notes"
				});

				expect(view.templateHelpers().getBookingNotes()).toEqual("Some Notes");
			});

			describe("Email Validation", function () {
				var email1 = "",
					email2 = "";

				it("shows an error if both emails are blank", function () {
					view.ui.$email.val(email1);
					view.ui.$confirmEmail.val(email2);

					view.validateEmail();

					expect(view.ui.$validator.eq(0).hasClass('field-validation-error')).toBeTruthy();
					expect(view.ui.$confirmEmailWrapper.hasClass('field-validation-error')).toBeFalsy();
					expect(view.ui.$confirmEmailWrapper.hasClass('disabled')).toBeTruthy();
					expect(view.ui.$okBtn[0].disabled).toBeTruthy();
				});

				it("shows an error if the email is invalid", function () {
					email1 = "test@test";

					view.ui.$email.val(email1);

					view.validateEmail();

					expect(view.ui.$validator.eq(0).hasClass('field-validation-error')).toBeTruthy();
					expect(view.ui.$confirmEmailWrapper.hasClass('field-validation-error')).toBeFalsy();
					expect(view.ui.$confirmEmailWrapper.hasClass('disabled')).toBeTruthy();
					expect(view.ui.$okBtn[0].disabled).toBeTruthy();
				});

				it("shows two errors if the email is invalid and the emails do not match", function () {
					email2 = "test";

					view.ui.$confirmEmail.val(email2);

					view.validateEmail();

					expect(view.ui.$validator.eq(0).hasClass('field-validation-error')).toBeTruthy();
					expect(view.ui.$confirmEmailWrapper.hasClass('field-validation-error')).toBeTruthy();
					expect(view.ui.$confirmEmailWrapper.hasClass('disabled')).toBeTruthy();
					expect(view.ui.$okBtn[0].disabled).toBeTruthy();
				});

				it("shows an error if the emails do not match", function () {
					email1 = "test@test.com";

					view.ui.$email.val(email1);

					view.validateEmail();

					expect(view.ui.$validator.eq(0).hasClass('field-validation-error')).toBeFalsy();
					expect(view.ui.$confirmEmailWrapper.hasClass('field-validation-error')).toBeTruthy();
					expect(view.ui.$confirmEmailWrapper.hasClass('disabled')).toBeFalsy();
					expect(view.ui.$okBtn[0].disabled).toBeTruthy();
				});

				it("enables the OK button if the emails match and are valid", function () {
					email2 = "test@test.com";

					view.ui.$confirmEmail.val(email2);

					view.validateEmail();

					expect(view.ui.$validator.eq(0).hasClass('field-validation-error')).toBeFalsy();
					expect(view.ui.$confirmEmailWrapper.hasClass('field-validation-error')).toBeFalsy();
					expect(view.ui.$confirmEmailWrapper.hasClass('disabled')).toBeFalsy();
					expect(view.ui.$okBtn[0].disabled).toBeFalsy();
				});
			});
		});

	}
);